fs = require 'fs'
path = require 'path'

parseComment = (commentStr) ->
    commentADT =
        directions: ""
        param: []
        return: {}
        usecases: []

    commentBlocks = commentStr.split "* @"
    if commentBlocks.length is 1
        context = commentBlocks[0].split(" *")
        return {
            directions: context[1] or context[0]
        }
    for commentBlock in commentBlocks
        commentBlock = commentBlock.trim()
        commentFlagLength = (commentBlock.match /\S+/)[0].length
        commentFlag = commentBlock.substr 0, commentFlagLength
        commentContent = commentBlock.substr commentFlagLength + 1

        switch do commentFlag.toLowerCase
            when '*'
                commentADT.directions = commentContent

            when 'param' or 'arg' or 'argu' or 'argument' or 'arguments'
                commentContent = commentContent.replace /\*\s+(.+)/, (arg...) ->
                        return arg[1]
                    .replace "\n", ""

                est = commentContent.match /\s?{(.+)}\s+(\S+?)\s+(.+)/

                commentADT.param.push
                    type: est[1]
                    name: est[2]
                    directions: est[3]

            when 'return'
                est = commentContent.match /\s?{(.+?)}\s+(.+)/
                commentADT.return =
                    type: est[1]
                    directions: est[2]

            when 'usecases' or 'usecase'
                for i in commentContent.split "* -"
                    if i.length isnt 0
                        commentADT.usecases.push i.trim()

    return commentADT

parseDefineInfo = (defineLine) ->
    classInfo = defineLine.match(/class\s+(\S+)/)
    funcInfo = defineLine.match(/(\S+).+?[-|=]>/)

    if classInfo
        name = classInfo[1]
        type = "Class"
    else if funcInfo
        name = funcInfo[1]
        type = "Function"

    return [name, type]


getCommentBlock = (source, index) ->

    while index < source.length

        fileLine = source[index]
        result = fileLine.match(/(\s+)###*/)

        if result?
            startLineNum = index
            startSpaceNum = result[1].length
            break
        index += 1

    while index < source.length
        index += 1
        fileLine = source[index]
        result = fileLine.match(new RegExp "\\s{" + startSpaceNum + "}###[^*]?")

        if result?
            defineInfo = parseDefineInfo source[startLineNum - 1]
            if defineInfo?
                block = (source.slice startLineNum + 1, index).join "\n"
                return {
                    commentBlock: block
                    name: defineInfo[0]
                    type: defineInfo[1]
                    commentLastIndex: index + 1
                }

        index += 1


exports.parse = (file) ->

    allCommentObj = []
    source = fs.readFileSync(file, 'utf-8').split('\n')
    i = 0

    while i < source.length
        result = getCommentBlock(source, i)

        if result?
            commentObj = parseComment result.commentBlock

            if commentObj?
                commentObj.type = result.type
                commentObj.name = result.name
                commentObj.from = path.basename file

                allCommentObj.push commentObj

            i = result.commentLastIndex
        else
            i += 1

    return allCommentObj